/*
 * Copyright 2017, OnGres.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following
 * disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


package com.ongres.scram.common.util;


import org.junit.Test;

import java.security.SecureRandom;
import java.util.Random;

import static org.junit.Assert.fail;


public class CryptoUtilTest {
    private static final SecureRandom SECURE_RANDOM = new SecureRandom();

    @Test(expected = IllegalArgumentException.class)
    public void nonceInvalidSize1() {
        CryptoUtil.nonce(0, SECURE_RANDOM);
    }

    @Test(expected = IllegalArgumentException.class)
    public void nonceInvalidSize2() {
        CryptoUtil.nonce(-1, SECURE_RANDOM);
    }

    @Test
    public void nonceValid() {
        int nNonces = 1000;
        int nonceMaxSize = 100;
        Random random = new Random();

        // Some more random testing
        for(int i = 0; i < nNonces; i++) {
            for(char c : CryptoUtil.nonce(random.nextInt(nonceMaxSize) + 1, SECURE_RANDOM).toCharArray()) {
                if(c == ',' || c < (char) 33 || c > (char) 126) {
                    fail("Character c='" + c + "' is not allowed on a nonce");
                }
            }
        }
    }
}
